

// this creates a case for a LED matrix based
// display that can be hung on a wall or set
// upright on its lower frame member

// the case is a stack of layers, each of which has a particular
// function in the whole, this is the stack from the 'front' to the 'back'

// diffusor
// front holes pattern
// led matrix layer
// baseplate (all electronics mount to this plate)
// space (hollow space, should be high enough to accomodate electronics, batteries etc)
// backplate

// setting a layer to '0' eliminates that particular layer

// TODO
// 19/Dec/2023 Initial working proof-of-concept
//             Much better kerf management
//             centering of the tabs (for some reason this is a lot harder than I thought it would be)
//             Cut wiring holes in base for the display
//             mark display positions on the base using engraving
//             Cut holes in the sides (the short pieces) to accomodate the joystick connectors
//             Cut a hole in one side to allow passing through the usb connector wire
//             Make diffusor optional (and respond to diffusor = 0) by removing the related features
//             in other parts
// 

// Some of the code below was lifted from:
// https://github.com/bmsleight/lasercut
// with the following license:

/*

Copyright (c) 2015, bmsleight
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

* Redistributions of source code must retain the above copyright notice, this
  list of conditions and the following disclaimer.

* Redistributions in binary form must reproduce the above copyright notice,
  this list of conditions and the following disclaimer in the documentation
  and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

// This code saved a lot of time and I'm grateful to the author for providing
// it but given the peculiarities of OpenSCAD and my laser cutter (which is a 
// cheap Chinese unit that isn't always as precise as I would like it to be
// kerf management, part alignment and general fit leave a lot ot be desired,
// I will re-do this part once I've figure out how to properly manage this (and
// make a couple of scale models to save some forests). But for now at least 
// I have a prototype to play with.

// I've stripped a lot of the code that I didn't need mostly related to various
// fixtures and ways to join pieces to get to the code stripped down to where
// I could follow better what was going on.

UP = 0;
DOWN = 180;
LEFT = 90;
RIGHT = 270;
MID = 360;
kerf=0.0;// Hacky global for kerf

module fingers(angle, start_up, fingers, thickness, range_min, range_max, t_x, t_y)
{

    // range_min = range_min * 1.1;
    // range_max = range_max / 1.1;

    // The tweaks to y translate([0, -thickness,0]) ... thickness*2 rather than *1
    // Are to avoid edge cases and make the dxf export better.
    // All fun
    
    range_min = range_min + thickness;
    range_max = range_max - thickness; 
    
    translate([t_x, t_y,0]) rotate([0,0,angle]) translate([0, -thickness,0])
    {
        for ( p = [ 0 : 1 : fingers-1] ) {
                i=range_min + ((range_max-range_min)/fingers)*p;
                if(start_up == 1) {
                        kerfSize = (p == 0) ? kerf/2 : kerf;
                        kerfMove = (p == 0) ? 0 : kerf/2;
                        translate([i-kerfMove,0,0]) 
                        {
                                cube([ (range_max-range_min)/(fingers*2) + kerfSize, 0.0+thickness*2, thickness]);
                }
            }
            else 
            {
                                kerfSize = (p==fingers-1) ? kerf/2 : kerf;
                                kerfMove = kerf/2;
                translate([i+(range_max-range_min)/(fingers*2)-kerfMove,thickness,0]) 
                {
                    cube([ (range_max-range_min)/(fingers*2)+kerfSize, thickness+0.01, thickness]);
                }
            }
        }
    }

}


module fingerJoint(angle, start_up, fingers, thickness, max_y, min_y, max_x, min_x)
{
    if ( angle == UP )
    {
        range_min = min_x; 
        range_max = max_x; 
        t_x = min_x;
        t_y = max_y;
        fingers(angle, start_up, fingers, thickness, range_min, range_max, t_x, t_y);
    }
    if ( angle == DOWN )
    {
        range_min = min_x; 
        range_max = max_x; 
        t_x = max_x;
        t_y = min_y;
        fingers(angle, start_up, fingers, thickness, range_min, range_max, t_x, t_y);
    }
    if ( angle == LEFT )
    {
        range_min = min_y; 
        range_max = max_y; 
        t_x = min_x;
        t_y = min_y;
        fingers(angle, start_up, fingers, thickness, range_min, range_max, t_x, t_y);
    }
    if ( angle == RIGHT )
    {
        range_min = min_y; 
        range_max = max_y; 
        t_x = max_x;
        t_y = max_y;
        fingers(angle, start_up, fingers, thickness, range_min, range_max, t_x, t_y);
    }
}



module simpleTab(angle, x, y, thickness)
{
    translate([x,y,0]) rotate([0,0,angle]) translate([-thickness/2,0,0]) cube([thickness, thickness, thickness]); 
}

module circlesRemove(radius, x, y, thickness)
{
    translate([x,y,-thickness]) cylinder(h=thickness*3, r=radius);
}

module lasercutout(thickness,  points= [], 
        simple_tabs=[],
        finger_joints = [],
        circles_remove = [],

)
{
    function max_y(points) = max([for (a = [0:1:len(points)-1])  points[a][1]]);    
    function min_y(points) = min([for (a = [0:1:len(points)-1])  points[a][1]]);    
    function max_x(points) = max([for (a = [0:1:len(points)-1])  points[a][0]]);    
    function min_x(points) = min([for (a = [0:1:len(points)-1])  points[a][0]]);    
    
    max_y = max_y(points);
    min_y = min_y(points);
    max_x = max_x(points);
    min_x = min_x(points);
    
    difference() 
    { 
        union() 
        {
            linear_extrude(height = thickness , center = false)  polygon(points=points);
            if(simple_tabs != undef) for (t = [0:1:len(simple_tabs)-1]) 
            {
                simpleTab(simple_tabs[t][0], simple_tabs[t][1], simple_tabs[t][2], thickness);
            }
            if(finger_joints != undef) for (t = [0:1:len(finger_joints)-1]) 
            {
                fingerJoint(finger_joints[t][0], finger_joints[t][1], finger_joints[t][2], thickness, max_y, min_y, max_x, min_x);
            }    
        } // end union

        if(circles_remove != undef) for (t = [0:1:len(circles_remove)-1]) {
            circlesRemove(circles_remove[t][0], circles_remove[t][1], circles_remove[t][2], thickness);
        }    
        
    }
   
}


module lasercutoutSquare(thickness, x=0, y=0, 
        simple_tabs=[],
        finger_joints = [],
        circles_remove = [],
)
{
    points = [[0,0], [x,0], [x,y], [0,y], [0,0]];
    lasercutout(thickness=thickness, points = points, simple_tabs = simple_tabs, finger_joints = finger_joints, circles_remove = circles_remove);
}


module display_stack(thickness, x, y, z, num_fingers, diffusor,holes,holepattern,leds,baseplate,space,backplate)

{
    
    fj4 = [ [UP, 0, num_fingers], [DOWN, 1, num_fingers], [LEFT, 0, num_fingers], [RIGHT, 1, num_fingers]  ];

    // this is the backplate
                
    color("Orange",1)
    translate([0,0,-5]) 
    // rotate([0,-90,0]) 
    lasercutoutSquare(thickness=thickness,x=z, y=y, finger_joints = fj4);
                             
    // this is the baseplate  
    color("Purple",1)
    translate([0,y+20,-5]) 
    // rotate([0,-90,0]) 
    lasercutoutSquare(thickness=thickness,x=z, y=y, finger_joints = fj4);
    
    // this is the optional LED holes layer just before the diffusor. If you don't
    // want to use it then setting it to zero should remove it. 
     
    color("Red",1)
    translate([0,2*y+40,-5]) 
    // rotate([0,-90,0]) 
    lasercutoutSquare(thickness=thickness,x=z, y=y, finger_joints = fj4, circles_remove = holepattern);    

    // the sides of the case
  
    tabs0 = [[DOWN, -thickness/2, 0], [DOWN, thickness-thickness/2, 0],[DOWN, x-thickness/2, 0], [DOWN, -thickness/2, thickness], [DOWN, -thickness/2, y+thickness], [DOWN, -thickness/2, y], [DOWN, thickness/2, y+thickness], [DOWN, x-thickness/2, y+thickness] ];
    fj0 = [[UP, 0, num_fingers], [DOWN, 1, num_fingers], [LEFT, 1, num_fingers]];
        
    // 'short' sides
        
    for (short=[0:1]) {    
    color("Green",1)  
    translate([short*(x+10),3*y+60,-5])
    difference() { 
        lasercutoutSquare(thickness=thickness,x=x, y=y, simple_tabs = tabs0, finger_joints = fj0);
        
        // this is really how it should work with the baseplate as a stencil
        // for all of the slots for tabs
        
        // rotate([0,90,0]) 
        // translate([0,0,-thickness])
        // lasercutoutSquare(thickness=thickness,x=z, y=y, finger_joints = fj4);
                
         rotate([0,90,0]) 
        translate([0,0,x-diffusor-thickness])
         lasercutoutSquare(thickness=thickness,x=z, y=y, finger_joints = fj4);
        
        // we use a copy of the base plate to cut the tab holes
        
        rotate([0,90,0])
        translate([0,0,x-diffusor-thickness-holes-leds])
        lasercutoutSquare(thickness=thickness,x=z, y=y, finger_joints = fj4);        
    }
}
            
    // 'long' sides

    tabs2 = [[DOWN, -thickness/2, thickness],[DOWN, -thickness/2, z],  ];
    fj2 = [ [UP, 1, num_fingers], [DOWN, 0, num_fingers], [LEFT, 1, num_fingers],   ];


    for (long=[2:3]) {
    color("LightBLUE",1) 
    translate([long*(x+10),3*y+60,-5]) 

    difference() {
     
         lasercutoutSquare(thickness=thickness,x=x, y=z, simple_tabs = tabs2, finger_joints = fj2);
  
         translate([x-diffusor,0,0])
         rotate([270,0,0])
        
         rotate([0,-90,0]) 
         
         lasercutoutSquare(thickness=thickness,x=z, y=y, finger_joints = fj4);

        translate([x-diffusor-holes-leds,0,0])
        
         rotate([270,0,0])
         rotate([0,-90,0]) 
        
        lasercutoutSquare(thickness=thickness,x=z, y=y, finger_joints = fj4);        
             
    }
    }
  
}

module layered_box(thickness, x, y, z, num_fingers, diffusor,holes,holepattern,leds,baseplate,space,backplate)
{
     
    display_stack(thickness,x-thickness,y-thickness*2 ,z-thickness*2,num_fingers, diffusor,holes,holepattern,leds, baseplate,space, backplate);    
}


module led_array_display(ledpitch, nx, ny, w, h,frame, diffusor,holes,leds,baseplate,space,backplate ) {

    stack = diffusor+holes+leds+baseplate+space+backplate;

    x = stack;
    
    y = 2.0 + ledpitch * h * ny + 2 * frame;
    z = 2.0 + ledpitch * w * nx + 2 * frame;
            
    ledholepattern = [ for (i= [0: 1 : (h*ny)-1]) for (j= [0: 1 : (w*nx)-1])
           [ 2.5, 1.0 + ledpitch / 2 + j * ledpitch, 1.0 + ledpitch / 2 + i * ledpitch]    
    ];

    // we generated the box pieces and then we slice right underneath and above them
    // to make sure that all of the cutouts are clean
    
    difference() {    
          layered_box(frame, x, y, z, 4, diffusor,holes,ledholepattern,leds,baseplate,space,backplate);
        
          translate([0,0,-1])
          linear_extrude(height = 10 , center = false)  polygon(points=[[-10000,-10000],[-10000,10000],[100000,10000],[10000,-10000]]);
        
           translate([0,0,-12]) 
        
           linear_extrude(height = 10 , center = false)  polygon(points=[[-10000,-10000],[-10000,10000],[100000,10000],[10000,-10000]]);
    }
}

projection(cut=false)
led_array_display(10.0, 6, 1, 8, 32,frame = 5.0, diffusor=5.0,holes=5.0,leds=3.0,baseplate=5.0,space=20,backplate=8);



